@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2014 2ndQuadrant Italia (Devise.IT S.r.L.)
+# Copyright (C) 2011-2015 2ndQuadrant Italia (Devise.IT S.r.L.)
 #
 # This file is part of Barman.
 #
@@ -21,6 +21,7 @@ This module represents a backup.
 
 from glob import glob
 import datetime
+import errno
 from io import StringIO
 import logging
 import os
@@ -35,10 +36,11 @@ import dateutil.tz
 from barman.infofile import WalFileInfo, BackupInfo, UnknownBackupIdException
 from barman.fs import UnixLocalCommand, UnixRemoteCommand, FsOperationFailed
 from barman import xlog, output
-from barman.command_wrappers import Rsync, RsyncPgData, CommandFailedException
+from barman.command_wrappers import Rsync, RsyncPgData, \
+    CommandFailedException
 from barman.compression import CompressionManager, CompressionIncompatibility
 from barman.hooks import HookScriptRunner
-from barman.utils import human_readable_timedelta, mkpath
+from barman.utils import human_readable_timedelta, mkpath, pretty_size
 from barman.config import BackupOptions
 
 
@@ -172,18 +174,23 @@ class BackupManager(object):
         """
         available_backups = self.get_available_backups()
         # Honour minimum required redundancy
-        if backup.status == BackupInfo.DONE and self.server.config.minimum_redundancy >= len(available_backups):
-            yield "Skipping delete of backup %s for server %s due to minimum redundancy requirements (%s)" % (
-                backup.backup_id, self.config.name, self.server.config.minimum_redundancy)
-            _logger.warning("Could not delete backup %s for server %s - minimum redundancy = %s, current size = %s"
-                % (backup.backup_id, self.config.name, self.server.config.minimum_redundancy, len(available_backups)))
+        if backup.status == BackupInfo.DONE and \
+                self.server.config.minimum_redundancy >= len(available_backups):
+            output.warning("Skipping delete of backup %s for server %s due to "
+                           "minimum redundancy requirements "
+                           "(minimum redundancy = %s, current redundancy = %s)",
+                           backup.backup_id,
+                           self.config.name,
+                           len(available_backups),
+                           self.server.config.minimum_redundancy)
             return
 
-        yield "Deleting backup %s for server %s" % (backup.backup_id, self.config.name)
+        output.info("Deleting backup %s for server %s",
+                    backup.backup_id, self.config.name)
         previous_backup = self.get_previous_backup(backup.backup_id)
         next_backup = self.get_next_backup(backup.backup_id)
-        # remove the backup
-        self.delete_basebackup(backup)
+        # Remove the pgdata directory of the backup
+        self.delete_pgdata(backup)
         # We are deleting the first available backup
         if not previous_backup:
             # In the case of exclusive backup (default), removes any WAL
@@ -197,10 +204,13 @@ class BackupManager(object):
                 remove_until = next_backup
             elif BackupOptions.CONCURRENT_BACKUP in self.config.backup_options:
                 remove_until = backup
-            yield "Delete associated WAL segments:"
+            output.info("Delete associated WAL segments:")
             for name in self._remove_unused_wal_files(remove_until):
-                yield "\t%s" % name
-        yield "Done"
+                output.info("\t%s", name)
+        # As last action, remove the basebackup directory,
+        # ending the delete operation
+        self.delete_basebackup(backup)
+        output.info("Done")
 
     def retry_backup_copy(self, target_function, *args, **kwargs):
         """
@@ -281,10 +291,26 @@ class BackupManager(object):
                 self.current_action = "copying files"
                 output.info("Copying files.")
                 # perform the backup copy, honouring the retry option if set
-                backup_size = self.retry_backup_copy(self.backup_copy,
-                                                     backup_info)
-                backup_info.set_attribute("size", backup_size)
+                self.retry_backup_copy(self.backup_copy, backup_info)
+                # Calculate deduplication ratio
+                if backup_info.size > 0:
+                    deduplication_ratio = 1 - (float(backup_info.deduplicated_size) /
+                                               backup_info.size)
+                else:
+                    deduplication_ratio = 0
+
                 output.info("Copy done.")
+                if self.config.reuse_backup == 'link':
+                    output.info(
+                        "Backup size: %s. Actual size on disk: %s"
+                        " (-%s deduplication ratio)." % (
+                        pretty_size(backup_info.size),
+                        pretty_size(backup_info.deduplicated_size),
+                        '{percent:.2%}'.format(percent=deduplication_ratio)
+                        ))
+                else:
+                    output.info("Backup size: %s" %
+                                pretty_size(backup_info.size))
             except:
                 # we do not need to do anything here besides re-throwin the
                 # exception. It will be handled in the external try block.
@@ -315,7 +341,7 @@ class BackupManager(object):
                     "failure %s (%s)" % (
                         self.current_action, msg_lines[0]))
 
-            output.exception("Backup failed %s: %s\n%s",
+            output.error("Backup failed %s.\nDETAILS: %s\n%s",
                          self.current_action, msg_lines[0],
                          '\n'.join(msg_lines[1:]))
 
@@ -371,31 +397,25 @@ class BackupManager(object):
             except FsOperationFailed:
                 output.error(
                     "Unable to connect to the target host using the command "
-                    "'%s'" % remote_command
-                )
-                return
+                    "'%s'" % remote_command)
+                output.close_and_exit()
         else:
             # if is a local recovery create a UnixLocalCommand
             cmd = UnixLocalCommand()
             # silencing static analysis tools
             rsync = None
-        msg = "Starting %s restore for server %s using backup %s " % (
-            recovery_dest, self.config.name, backup.backup_id)
-        yield msg
-        _logger.info(msg)
+        output.info("Starting %s restore for server %s using backup %s ",
+                    recovery_dest, self.config.name, backup.backup_id)
 
         # check destination directory. If doesn't exist create it
         try:
             cmd.create_dir_if_not_exists(dest)
         except FsOperationFailed, e:
-            msg = ("unable to initialize destination directory "
-                   "'%s': %s" % (dest, e))
-            _logger.exception(msg)
-            raise SystemExit(msg)
+            output.exception("unable to initialize destination directory "
+                             "'%s': %s", dest, e)
+            output.close_and_exit()
 
-        msg = "Destination directory: %s" % dest
-        yield msg
-        _logger.info(msg)
+        output.info("Destination directory: %s", dest)
 
         # initialize tablespace structure
         if backup.tablespaces:
@@ -405,10 +425,9 @@ class BackupManager(object):
                 # if does not exists, create it
                 cmd.create_dir_if_not_exists(tblspc_dir)
             except FsOperationFailed, e:
-                msg = ("unable to initialize tablespace directory "
-                       "'%s': %s" % (tblspc_dir, e))
-                _logger.exception(msg)
-                raise SystemExit(msg)
+                output.exception("unable to initialize tablespace directory "
+                                 "'%s': %s", tblspc_dir, e)
+                output.close_and_exit()
 
             for item in backup.tablespaces:
 
@@ -436,33 +455,32 @@ class BackupManager(object):
                     # create symlink between tablespace and recovery folder
                     cmd.create_symbolic_link(location, pg_tblspc_file)
                 except FsOperationFailed, e:
-                    msg = ("unable to prepare '%s' tablespace "
-                           "(destination '%s'): %s" %
-                           (item.name, location, e))
-                    _logger.exception(msg)
-                    raise SystemExit(msg)
+                    output.exception("unable to prepare '%s' tablespace "
+                                     "(destination '%s'): %s",
+                                     item.name, location, e)
+                    output.close_and_exit()
 
-                yield "\t%s, %s, %s" % (item.oid, item.name, location)
+                output.info("\t%s, %s, %s", item.oid, item.name, location)
 
         wal_dest = os.path.join(dest, 'pg_xlog')
         target_epoch = None
         target_datetime = None
         if target_time:
+            # noinspection PyBroadException
             try:
                 target_datetime = dateutil.parser.parse(target_time)
             except ValueError as e:
-                msg = "unable to parse the target time parameter %r: %s" % (
-                      target_time, e)
-                _logger.exception(msg)
-                raise SystemExit(msg)
+                output.exception("unable to parse the target time parameter "
+                                 "%r: %s", target_time, e)
+                output.close_and_exit()
             except Exception:
                 # this should not happen, but there is a known bug in
                 # dateutil.parser.parse() implementation
                 # ref: https://bugs.launchpad.net/dateutil/+bug/1247643
-                msg = "unable to parse the target time parameter %r" % (
-                      target_time)
-                _logger.exception(msg)
-                raise SystemExit(msg)
+                output.exception("unable to parse the target time parameter "
+                                 "%r", target_time)
+                output.close_and_exit()
+
             target_epoch = time.mktime(target_datetime.timetuple()) + (
                 target_datetime.microsecond / 1000000.)
         if target_time or target_xid or (
@@ -476,8 +494,9 @@ class BackupManager(object):
                 targets['timeline'] = str(target_tli)
             if target_name:
                 targets['name'] = str(target_name)
-            yield "Doing PITR. Recovery target %s" % \
-                (", ".join(["%s: %r" % (k, v) for k, v in targets.items()]))
+            output.info(
+                "Doing PITR. Recovery target %s",
+                (", ".join(["%s: %r" % (k, v) for k, v in targets.items()])))
             wal_dest = os.path.join(dest, 'barman_xlog')
 
         # Retrieve the safe_horizon for smart copy
@@ -525,25 +544,22 @@ class BackupManager(object):
             self.retry_backup_copy(self.recover_basebackup_copy, backup, dest,
                                    tablespaces, remote_command, safe_horizon)
         except DataTransferFailure, e:
-            raise SystemExit("Failure copying base backup: %s" % (e,))
-        _logger.info("Base backup copied.")
+            output.exception("Failure copying base backup: %s", e)
+            output.close_and_exit()
 
         # Prepare WAL segments local directory
-        msg = "Copying required wal segments."
-        _logger.info(msg)
-        yield msg
+        output.info("Copying required wal segments.")
 
         # Retrieve the list of required WAL segments
         # according to recovery options
         xlogs = {}
         required_xlog_files = tuple(
-            self.server.get_required_xlog_files(backup, target_tli,
-                                                target_epoch))
-        for filename in required_xlog_files:
-            hashdir = xlog.hash_dir(filename)
+            self.server.get_required_xlog_files(backup, target_tli, target_epoch))
+        for wal_info in required_xlog_files:
+            hashdir = xlog.hash_dir(wal_info.name)
             if hashdir not in xlogs:
                 xlogs[hashdir] = []
-            xlogs[hashdir].append(filename)
+            xlogs[hashdir].append(wal_info.name)
         # Check decompression options
         compressor = self.compression_manager.get_compressor()
 
@@ -551,14 +567,13 @@ class BackupManager(object):
         try:
             self.recover_xlog_copy(compressor, xlogs, wal_dest, remote_command)
         except DataTransferFailure, e:
-            raise SystemExit("Failure copying WAL files: %s" % (e,))
+            output.exception("Failure copying WAL files: %s", e)
+            output.close_and_exit()
 
         # Generate recovery.conf file (only if needed by PITR)
         if target_time or target_xid or (
                 target_tli and target_tli != backup.timeline) or target_name:
-            msg = "Generating recovery.conf"
-            yield msg
-            _logger.info(msg)
+            output.info("Generating recovery.conf")
             if remote_command:
                 tempdir = tempfile.mkdtemp(prefix='barman_recovery-')
                 recovery = open(os.path.join(tempdir, 'recovery.conf'), 'w')
@@ -589,22 +604,21 @@ class BackupManager(object):
                     plain_rsync.from_file_list(['recovery.conf'],
                                               tempdir, ':%s' % dest)
                 except CommandFailedException, e:
-                    msg = (
-                        'remote copy of recovery.conf failed: %s' % (e,))
-                    _logger.exception(msg)
-                    raise SystemExit(msg)
+                    output.exception(
+                        'remote copy of recovery.conf failed: %s', e)
+                    output.close_and_exit()
 
                 shutil.rmtree(tempdir)
-            _logger.info('recovery.conf generated')
         else:
             # avoid shipping of just recovered pg_xlog files
+            output.info("Generating archive status files")
             if remote_command:
                 status_dir = tempfile.mkdtemp(prefix='barman_xlog_status-')
             else:
                 status_dir = os.path.join(wal_dest, 'archive_status')
                 mkpath(status_dir)
-            for filename in required_xlog_files:
-                with open(os.path.join(status_dir, "%s.done" % filename),
+            for wal_info in required_xlog_files:
+                with open(os.path.join(status_dir, "%s.done" % wal_info.name),
                           'a') as f:
                     f.write('')
             if remote_command:
@@ -612,12 +626,15 @@ class BackupManager(object):
                     rsync('%s/' % status_dir,
                           ':%s' % os.path.join(wal_dest, 'archive_status'))
                 except CommandFailedException:
-                    msg = "unable to populate pg_xlog/archive_status directory"
-                    _logger.warning(msg, exc_info=1)
-                    raise SystemExit(msg)
+                    output.exception(
+                        "unable to populate pg_xlog/archive_status directory",
+                        e)
+                    output.close_and_exit()
+
                 shutil.rmtree(status_dir)
 
         # Disable dangerous setting in the target data dir
+        output.info("Disabling dangerous settings in destination directory.")
         if remote_command:
             tempdir = tempfile.mkdtemp(prefix='barman_recovery-')
             pg_config = os.path.join(tempdir, 'postgresql.conf')
@@ -629,10 +646,8 @@ class BackupManager(object):
         if self.pg_config_mangle(pg_config,
                                  {'archive_command': 'false'},
                                  "%s.origin" % pg_config):
-            msg = "The archive_command was set to 'false' to prevent data " \
-                  "losses."
-            yield msg
-            _logger.info(msg)
+            output.info("The archive_command was set to 'false' "
+                        "to prevent data losses.")
 
         # Find dangerous options in the configuration file (locations)
         clashes = self.pg_config_detect_possible_issues(pg_config)
@@ -643,9 +658,9 @@ class BackupManager(object):
                     ['postgresql.conf', 'postgresql.conf.origin'], tempdir,
                     ':%s' % dest)
             except CommandFailedException, e:
-                msg = 'remote copy of configuration files failed: %s' % (e,)
-                _logger.error(msg)
-                raise SystemExit(msg)
+                output.exception(
+                    'remote copy of configuration files failed: %s', e)
+                output.close_and_exit()
             shutil.rmtree(tempdir)
 
         # Copy the backup.info file to the destination as ".barman-recover.info"
@@ -653,42 +668,44 @@ class BackupManager(object):
             try:
                 rsync(backup.filename, ':%s/.barman-recover.info' % dest)
             except CommandFailedException, e:
-                    msg = 'copy of recovery metadata file failed: %s' % (e,)
-                    _logger.error(msg)
-                    raise SystemExit(msg)
+                    output.exception(
+                        'copy of recovery metadata file failed: %s', e)
+                    output.close_and_exit()
         else:
             backup.save(os.path.join(dest, '.barman-recover.info'))
 
-        yield ""
-        yield "Your PostgreSQL server has been successfully prepared for " \
-              "recovery!"
-        yield ""
-        yield "Please review network and archive related settings in the " \
-              "PostgreSQL"
-        yield "configuration file before starting the just recovered instance."
-        yield ""
+        output.info("", log=False)
+        output.info("Your PostgreSQL server has been successfully prepared for "
+              "recovery!", log=False)
+        output.info("", log=False)
+        output.info("Please review network and archive related settings in the "
+              "PostgreSQL", log=False)
+        output.info("configuration file before starting the just recovered "
+                    "instance.", log=False)
+        output.info("", log=False)
         # With a PostgreSQL version older than 8.4, it is the user's
         # responsibility to delete the "barman_xlog" directory as the
         # restore_command option in recovery.conf is not supported
         if backup.version < 80400 and (target_time or target_xid or (
                 target_tli and target_tli != backup.timeline) or target_name):
-            yield "After the recovery, please remember to remove the " \
-                  "\"barman_xlog\" directory"
-            yield "inside the PostgreSQL data directory."
-            yield ""
+            output.info("After the recovery, please remember to remove the "
+                  "\"barman_xlog\" directory", log=False)
+            output.info("inside the PostgreSQL data directory.", log=False)
+            output.info("", log=False)
         if clashes:
-            yield "WARNING: Before starting up the recovered PostgreSQL server,"
-            yield "please review also the settings of the following " \
-                  "configuration"
-            yield "options as they might interfere with your current " \
-                  "recovery attempt:"
-            yield ""
+            output.info("WARNING: Before starting up the recovered PostgreSQL "
+                        "server,", log=False)
+            output.info("please review also the settings of the following "
+                        "configuration", log=False)
+            output.info("options as they might interfere with your current "
+                        "recovery attempt:", log=False)
+            output.info("", log=False)
 
             for name, value in sorted(clashes.items()):
-                yield "    %s = %s" % (name, value)
+                output.info("    %s = %s", name, value, log=False)
 
-            yield ""
-        _logger.info("Recovery completed successful.")
+            output.info("", log=False)
+        output.info("Recovery completed successful.")
 
     def cron(self, verbose=True):
         """
@@ -706,8 +723,10 @@ class BackupManager(object):
                 output.info("Processing xlog segments for %s",
                             self.config.name,
                             log=False)
-            available_backups = self.get_available_backups(
-                BackupInfo.STATUS_ALL)
+            # Get the first available backup
+            first_backup_id = self.get_first_backup(BackupInfo.STATUS_ALL)
+            first_backup = self.server.get_backup(first_backup_id) \
+                    if first_backup_id else None
             for filename in sorted(glob(
                     os.path.join(self.config.incoming_wals_directory, '*'))):
                 if not found and not verbose:
@@ -715,22 +734,36 @@ class BackupManager(object):
                                 self.config.name,
                                 log=False)
                 found = True
-                # Delete xlog segments only if the backup is exclusive
-                if (not len(available_backups) and
-                        (BackupOptions.CONCURRENT_BACKUP not in
-                             self.config.backup_options)):
-                    output.info("\tNo base backup available. Trashing file %s"
+
+                # Create WAL Info object
+                wal_info = WalFileInfo.from_file(filename, compression=None)
+
+                # If there are no available backups ...
+                if first_backup is None:
+                    # ... delete xlog segments only for exclusive backups
+                    if BackupOptions.CONCURRENT_BACKUP not in self.config.backup_options:
+                        output.info("\tNo base backup available. Trashing file %s"
                                 " from server %s",
-                                os.path.basename(filename), self.config.name)
-                    os.unlink(filename)
-                    continue
+                                wal_info.name, self.config.name)
+                        os.unlink(filename)
+                        continue
+                # ... otherwise
+                else:
+                    # ... delete xlog segments older than the first backup
+                    if wal_info.name < first_backup.begin_wal:
+                        output.info("\tOlder than first backup. Trashing file %s"
+                                " from server %s",
+                                wal_info.name, self.config.name)
+                        os.unlink(filename)
+                        continue
+
                 # Report to the user the WAL file we are archiving
                 output.info("\t%s", os.path.basename(filename), log=False)
                 _logger.info("Archiving %s/%s",
                              self.config.name,
                              os.path.basename(filename))
                 # Archive the WAL file
-                wal_info = self.cron_wal_archival(compressor, filename)
+                self.cron_wal_archival(compressor, wal_info)
 
                 # Updates the information of the WAL archive with
                 # the latest segments
@@ -738,7 +771,10 @@ class BackupManager(object):
         if not found and verbose:
             output.info("\tno file found", log=False)
 
-        # Retention policy management
+    def cron_retention_policy(self):
+        """
+        Retention policy management
+        """
         if (self.server.enforce_retention_policies
                 and self.config.retention_policy_mode == 'auto'):
             available_backups = self.get_available_backups(
@@ -749,18 +785,29 @@ class BackupManager(object):
                     output.info(
                         "Enforcing retention policy: removing backup %s for "
                         "server %s" % (bid, self.config.name))
-                    for line in self.delete_backup(available_backups[bid]):
-                        output.info(line)
+                    self.delete_backup(available_backups[bid])
 
     def delete_basebackup(self, backup):
-        '''
-        Delete the given base backup
+        """
+        Delete the basebackup dir of a given backup.
 
-        :param backup: the backup to delete
-        '''
-        backup_dir = backup.get_basebackup_directory();
+        :param barman.infofile.BackupInfo backup: the backup to delete
+        """
+        backup_dir = backup.get_basebackup_directory()
+        _logger.debug("Deleting base backup directory: %s" % backup_dir)
         shutil.rmtree(backup_dir)
 
+    def delete_pgdata(self, backup):
+        """
+        Delete the pgdata dir of a given backup.
+
+        :param barman.infofile.BackupInfo backup: the backup to delete
+        """
+        backup_dir = backup.get_basebackup_directory()
+        pg_data = os.path.join(backup_dir, 'pgdata')
+        _logger.debug("Deleting PGDATA directory: %s" % pg_data)
+        shutil.rmtree(pg_data)
+
     def delete_wal(self, wal_info):
         """
         Delete a WAL segment, with the given name
@@ -769,9 +816,9 @@ class BackupManager(object):
         """
 
         try:
-            os.unlink(wal_info.full_path)
+            os.unlink(wal_info.fullpath(self.server))
             try:
-                os.removedirs(os.path.dirname(wal_info.full_path))
+                os.removedirs(os.path.dirname(wal_info.fullpath(self.server)))
             except OSError:
                 # This is not an error condition
                 # We always try to remove the the trailing directories,
@@ -787,12 +834,13 @@ class BackupManager(object):
 
         :param BackupInfo backup_info: the backup information
         """
-        self.current_action = "connecting to database (%s)" % self.config.conninfo
-        _logger.debug(self.current_action)
+        self.current_action = "connecting to database (%s)" % \
+                              self.config.conninfo
+        output.debug(self.current_action)
 
         # Set the PostgreSQL data directory
         self.current_action = "detecting data directory"
-        _logger.debug(self.current_action)
+        output.debug(self.current_action)
         data_directory = self.server.get_pg_setting('data_directory')
         backup_info.set_attribute('pgdata', data_directory)
 
@@ -807,7 +855,7 @@ class BackupManager(object):
 
         # Get tablespaces information
         self.current_action = "detecting tablespaces"
-        _logger.debug(self.current_action)
+        output.debug(self.current_action)
         tablespaces = self.server.get_pg_tablespaces()
         if tablespaces and len(tablespaces) > 0:
             backup_info.set_attribute("tablespaces", tablespaces)
@@ -877,6 +925,21 @@ class BackupManager(object):
                 if tablespace_data.location.startswith(backup_info.pgdata)
             ]
 
+        # Retrieve the previous backup metadata and set the safe_horizon
+        # accordingly
+        previous_backup = self.get_previous_backup(backup_info.backup_id)
+        if previous_backup:
+            # safe_horizon is a tz-aware timestamp because BackupInfo class
+            # ensures it
+            safe_horizon = previous_backup.begin_time
+        else:
+            # If no previous backup is present, the safe horizon is set to None
+            safe_horizon = None
+
+        # Make sure the destination directory exists in order for smart copy
+        # to detect that no file is present there
+        mkpath(backup_dest)
+
         # deal with tablespaces with a different bwlimit
         if len(tablespaces_bwlimit) > 0:
             # we are copying the tablespaces before the data directory,
@@ -886,29 +949,42 @@ class BackupManager(object):
                 self.current_action = "copying tablespace '%s' with bwlimit " \
                                       "%d" % (tablespace_dir, bwlimit)
                 _logger.debug(self.current_action)
+                ref_dir = self.reuse_dir(previous_backup, tablespace_dir)
                 tb_rsync = RsyncPgData(
                     ssh=self.server.ssh_command,
                     ssh_options=self.server.ssh_options,
+                    args=self.reuse_args(ref_dir),
                     bwlimit=bwlimit,
                     network_compression=self.config.network_compression,
                     check=True)
                 try:
-                    tb_rsync(':%s/' %
-                             os.path.join(backup_info.pgdata, tablespace_dir),
-                             os.path.join(backup_dest, tablespace_dir))
+                    tablespace_dest = os.path.join(backup_dest, tablespace_dir)
+                    # make sure the destination directory exists in order for
+                    # smart copy to detect that no file is present there
+                    mkpath(tablespace_dest)
+                    tb_rsync.smart_copy(
+                        ':%s/' % os.path.join(backup_info.pgdata,
+                                              tablespace_dir),
+                        tablespace_dest,
+                        safe_horizon,
+                        ref_dir)
                 except CommandFailedException, e:
                     msg = "data transfer failure on directory '%s'" % \
                           os.path.join(backup_info.pgdata, tablespace_dir)
                     self._raise_rsync_error(e, msg)
 
+        ref_dir = self.reuse_dir(previous_backup)
         rsync = RsyncPgData(
             ssh=self.server.ssh_command,
             ssh_options=self.server.ssh_options,
+            args=self.reuse_args(ref_dir),
             bwlimit=self.config.bandwidth_limit,
             exclude_and_protect=exclude_and_protect,
             network_compression=self.config.network_compression)
         try:
-            rsync(':%s/' % backup_info.pgdata, backup_dest)
+            rsync.smart_copy(':%s/' % backup_info.pgdata, backup_dest,
+                             safe_horizon,
+                             ref_dir)
         except CommandFailedException, e:
             msg = "data transfer failure on directory '%s'" % \
                   backup_info.pgdata
@@ -946,27 +1022,83 @@ class BackupManager(object):
                             rsync(':%s' % cf[key], backup_dest)
 
                         except CommandFailedException, e:
+                            ret_code = e.args[0]['ret']
                             msg = "data transfer failure on file '%s'" % \
                                   cf[key]
-                            self._raise_rsync_error(e, msg)
+                            if 'ident_file' == key and ret_code == 23:
+                                # if the ident file is not present
+                                # it is not a blocking error, so,
+                                # we need to track why the exception is raised.
+                                # if ident file is missing, warn the user, log
+                                # the data transfer but continue the backup
+                                output.warning(msg, log=True)
+                                continue
+                            else:
+                                self._raise_rsync_error(e, msg)
 
         # Calculate the base backup size
         self.current_action = "calculating backup size"
         _logger.debug(self.current_action)
         backup_size = 0
+        deduplicated_size = 0
         for dirpath, _, filenames in os.walk(backup_dest):
             # execute fsync() on the containing directory
             dir_fd = os.open(dirpath, os.O_DIRECTORY)
-            os.fsync(dir_fd)
+            try:
+                os.fsync(dir_fd)
+            except OSError, e:
+                # On some filesystem doing a fsync on a directory
+                # raises an EINVAL error. Ignoring it is usually safe.
+                if e.errno != errno.EINVAL:
+                    raise
             os.close(dir_fd)
             # execute fsync() on all the contained files
             for filename in filenames:
                 file_path = os.path.join(dirpath, filename)
                 file_fd = os.open(file_path, os.O_RDONLY)
-                backup_size += os.fstat(file_fd).st_size
+                file_stat = os.fstat(file_fd)
+                backup_size += file_stat.st_size
+                # Excludes hard links from real backup size
+                if file_stat.st_nlink == 1:
+                    deduplicated_size += file_stat.st_size
                 os.fsync(file_fd)
                 os.close(file_fd)
-        return backup_size
+        # Save size into BackupInfo object
+        backup_info.set_attribute('size', backup_size)
+        backup_info.set_attribute('deduplicated_size', deduplicated_size)
+
+    def reuse_dir(self, previous_backup_info, path=None):
+        """
+        If reuse_backup is 'copy' or 'link', builds the path of the directory
+        to reuse, otherwise always returns None.
+
+        If path is None, it returns the full path of pgdata directory of
+        the previous_backup otherwise it returns the specified path inside the
+        data directory.
+
+        :param BackupInfo previous_backup_info: backup to be reused
+        :param str path: path inside the data directory to be reused
+        :return str: the local path with data to be reused
+        """
+        if self.config.reuse_backup in ('copy', 'link') and previous_backup_info is not None:
+            last_backup_dir = previous_backup_info.get_basebackup_directory()
+            reuse_dir = os.path.join(last_backup_dir, 'pgdata')
+            if path is not None:
+                reuse_dir = os.path.join(reuse_dir, path)
+            return reuse_dir
+
+    def reuse_args(self, reuse_dir):
+        """
+        If reuse_backup is 'copy' or 'link', build the rsync option to enable
+        the reuse, otherwise returns an empty list
+
+        :param str|None reuse_dir: the local path with data to be reused
+        :return list: the argument list for rsync
+        """
+        if self.config.reuse_backup in ('copy', 'link') and reuse_dir is not None:
+            return ['--%s-dest=%s' % (self.config.reuse_backup, reuse_dir)]
+        else:
+            return []
 
     def backup_stop(self, backup_info):
         '''
@@ -1114,7 +1246,13 @@ class BackupManager(object):
 
                     # Cleanup files after the transfer
                     for segment in xlogs[prefix]:
-                        os.unlink(os.path.join(xlog_spool, segment))
+                        file_name = os.path.join(xlog_spool, segment)
+                        try:
+                            os.unlink(file_name)
+                        except OSError as e:
+                            output.warning(
+                                "Error removing temporary file '%s': %s",
+                                file_name, e)
                 else:
                     # decompress directly to the right place
                     for segment in xlogs[prefix]:
@@ -1137,54 +1275,56 @@ class BackupManager(object):
         if compressor and remote_command:
             shutil.rmtree(xlog_spool)
 
-    def cron_wal_archival(self, compressor, filename):
+    def cron_wal_archival(self, compressor, wal_info):
         """
         Archive a WAL segment from the incoming directory.
         This function returns a WalFileInfo object.
 
         :param compressor: the compressor for the file (if any)
-        :param filename: the name of the WAL file is being processed
-        :return WalFileInfo:
+        :param wal_info: WalFileInfo of the WAL file is being processed
         """
-        basename = os.path.basename(filename)
-        destdir = os.path.join(self.config.wals_directory, xlog.hash_dir(basename))
-        destfile = os.path.join(destdir, basename)
-
-        wal_info = WalFileInfo.from_file(filename, compression=None)
+        destfile = wal_info.fullpath(self.server)
+        destdir = os.path.dirname(destfile)
+        srcfile = os.path.join(self.config.incoming_wals_directory,
+                wal_info.name)
 
         # Run the pre_archive_script if present.
         script = HookScriptRunner(self, 'archive_script', 'pre')
-        script.env_from_wal_info(wal_info)
+        script.env_from_wal_info(wal_info, srcfile)
         script.run()
 
         mkpath(destdir)
         if compressor:
-            compressor.compress(filename, destfile)
-            shutil.copystat(filename, destfile)
-            os.unlink(filename)
+            compressor.compress(srcfile, destfile)
+            shutil.copystat(srcfile, destfile)
+            os.unlink(srcfile)
         else:
-            shutil.move(filename, destfile)
+            shutil.move(srcfile, destfile)
 
         # execute fsync() on the archived WAL containing directory
-        dir_fd = os.open(os.path.dirname(destfile), os.O_DIRECTORY)
-        os.fsync(dir_fd)
+        dir_fd = os.open(destdir, os.O_DIRECTORY)
+        try:
+            os.fsync(dir_fd)
+        except OSError, e:
+            # On some filesystem doing a fsync on a directory
+            # raises an EINVAL error. Ignoring it is usually safe.
+            if e.errno != errno.EINVAL:
+                raise
         os.close(dir_fd)
         # execute fsync() on the archived WAL file
         file_fd = os.open(destfile, os.O_RDONLY)
         os.fsync(file_fd)
         os.close(file_fd)
 
-        wal_info = WalFileInfo.from_file(
-            destfile,
-            compression=compressor and compressor.compression)
+        stat = os.stat(destfile)
+        wal_info.size = stat.st_size
+        wal_info.compression = compressor and compressor.compression
 
         # Run the post_archive_script if present.
         script = HookScriptRunner(self, 'archive_script', 'post')
         script.env_from_wal_info(wal_info)
         script.run()
 
-        return wal_info
-
     def check(self):
         """
         This function performs some checks on the server.
@@ -1311,7 +1451,7 @@ class BackupManager(object):
         """
         from os.path import isdir, join
 
-        yield "Rebuilding xlogdb for server %s" % self.config.name
+        output.info("Rebuilding xlogdb for server %s", self.config.name)
         root = self.config.wals_directory
         default_compression = self.config.compression
         wal_count = label_count = history_count = 0
@@ -1361,9 +1501,9 @@ class BackupManager(object):
                             'rebuilding the wal database: %s',
                             fullname)
 
-        yield 'Done rebuilding xlogdb for server %s ' \
-            '(history: %s, backup_labels: %s, wal_file: %s)' % (
-                self.config.name, history_count, label_count, wal_count)
+        output.info('Done rebuilding xlogdb for server %s '
+                    '(history: %s, backup_labels: %s, wal_file: %s)',
+                    self.config.name, history_count, label_count, wal_count)
 
     def _write_backup_label(self, backup_info):
         """
