@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2014 2ndQuadrant Italia (Devise.IT S.r.L.)
+# Copyright (C) 2011-2015 2ndQuadrant Italia (Devise.IT S.r.L.)
 #
 # This file is part of Barman.
 #
@@ -18,20 +18,22 @@
 """
 This module implements the interface with the command line and the logger.
 """
+import logging
+import os
+import sys
 
 from argh import ArghParser, named, arg, expects_obj
 from argparse import SUPPRESS, ArgumentTypeError
+
 from barman import output
 from barman.infofile import BackupInfo
 from barman import lockfile
 from barman.server import Server
 import barman.diagnose
 import barman.config
-import logging
-import os
-import sys
 from barman.utils import drop_privileges, configure_logging, parse_log_level
 
+
 _logger = logging.getLogger(__name__)
 
 
@@ -88,21 +90,18 @@ def cron():
 def server_completer(prefix, parsed_args, **kwargs):
     global_config(parsed_args)
     for conf in barman.__config__.servers():
-        if conf.name.startswith(prefix) \
-                and conf.name not in parsed_args.server_name:
+        if conf.name.startswith(prefix):
             yield conf.name
 
 
 # noinspection PyUnusedLocal
 def server_completer_all(prefix, parsed_args, **kwargs):
     global_config(parsed_args)
+    current_list = getattr(parsed_args, 'server_name', None) or ()
     for conf in barman.__config__.servers():
-        if conf.name.startswith(prefix) \
-                and conf.name not in parsed_args.server_name:
+        if conf.name.startswith(prefix) and conf.name not in current_list:
             yield conf.name
-    if 'server_name' in parsed_args \
-            and parsed_args.server_name is None \
-            and 'all'.startswith(prefix):
+    if len(current_list) == 0 and 'all'.startswith(prefix):
         yield 'all'
 
 
@@ -110,17 +109,15 @@ def server_completer_all(prefix, parsed_args, **kwargs):
 def backup_completer(prefix, parsed_args, **kwargs):
     global_config(parsed_args)
     server = get_server(parsed_args)
-    if server and len(parsed_args.backup_id) == 0:
-        for backup_id in sorted(server.get_available_backups().iterkeys(),
-                                reverse=True):
+    backup_name = getattr(parsed_args, 'backup_id', None) or ''
+    if server:
+        backups = server.get_available_backups()
+        for backup_id in sorted(backups, reverse=True):
             if backup_id.startswith(prefix):
                 yield backup_id
         for special_id in ('latest', 'last', 'oldest', 'first'):
-            if len(server.get_available_backups()) > 0 \
-                    and special_id.startswidth(prefix):
+            if len(backups) > 0 and special_id.startswith(prefix):
                 yield special_id
-    else:
-        return
 
 
 @arg('server_name', nargs='+',
@@ -137,6 +134,11 @@ def backup_completer(prefix, parsed_args, **kwargs):
      dest='immediate_checkpoint',
      action='store_false',
      default=SUPPRESS)
+@arg('--reuse-backup', nargs='?',
+     choices=barman.config.REUSE_BACKUP_VALUES,
+     default=None, const='link',
+     help='use the previous backup to improve transfer-rate. '
+          'If no argument is given "link" is assumed')
 @arg('--retry-times',
      help='Number of retries after an error if base backup copy fails.',
      type=check_positive)
@@ -156,6 +158,8 @@ def backup(args):
         if server is None:
             output.error("Unknown server '%s'" % name)
             continue
+        if args.reuse_backup is not None:
+            server.config.reuse_backup = args.reuse_backup
         if args.retry_sleep is not None:
             server.config.basebackup_retry_sleep = args.retry_sleep
         if args.retry_times is not None:
@@ -163,7 +167,6 @@ def backup(args):
         if hasattr(args, 'immediate_checkpoint'):
             server.config.immediate_checkpoint = args.immediate_checkpoint
         server.backup()
-
     output.close_and_exit()
 
 
@@ -217,18 +220,14 @@ def rebuild_xlogdb(args):
     Rebuild the WAL file database guessing it from the disk content.
     """
     servers = get_server_list(args)
-    ok = True
     for name in sorted(servers):
         server = servers[name]
         if server is None:
-            yield "Unknown server '%s'" % (name)
-            ok = False
+            output.error("Unknown server '%s'", name)
             continue
-        for line in server.rebuild_xlogdb():
-            yield line
-        yield ''
-    if not ok:
-        raise SystemExit(1)
+        server.rebuild_xlogdb()
+    output.close_and_exit()
+
 
 @arg('server_name',
      completer=server_completer,
@@ -250,6 +249,7 @@ def rebuild_xlogdb(args):
           'to be launched on a remote host. It is the equivalent of the "ssh_command" server'
           'option in the configuration file for remote recovery. Example: "ssh postgres@db2"')
 @arg('backup_id',
+     completer=backup_completer,
      help='specifies the backup ID to recover')
 @arg('destination_directory',
      help='the directory where the new server is created')
@@ -268,12 +268,14 @@ def recover(args):
     """
     server = get_server(args)
     if server is None:
-        raise SystemExit("ERROR: unknown server '%s'" % (args.server_name))
+        output.error("Unknown server '%s'", args.server_name)
+        output.close_and_exit()
     # Retrieves the backup
     backup = parse_backup_id(server, args)
     if backup is None or backup.status != BackupInfo.DONE:
-        raise SystemExit("ERROR: unknown backup '%s' for server '%s'" % (
-            args.backup_id, args.server_name))
+        output.error("Unknown backup '%s' for server '%s'",
+                     args.backup_id, args.server_name)
+        output.close_and_exit()
 
     # decode the tablespace relocation rules
     tablespaces = {}
@@ -282,41 +284,45 @@ def recover(args):
             try:
                 tablespaces.update([rule.split(':', 1)])
             except ValueError:
-                raise SystemExit(
-                    "ERROR: invalid tablespace relocation rule '%s'\n"
-                    "HINT: the valid syntax for a relocation rule is "
-                    "NAME:LOCATION" % rule)
+                output.error("Invalid tablespace relocation rule '%s'\n"
+                    "HINT: The valid syntax for a relocation rule is "
+                    "NAME:LOCATION", rule)
+                output.close_and_exit()
 
     # validate the rules against the tablespace list
     valid_tablespaces = [tablespace_data.name for tablespace_data in
                          backup.tablespaces] if backup.tablespaces else []
     for item in tablespaces:
         if item not in valid_tablespaces:
-            raise SystemExit("ERROR: invalid tablespace name '%s'\n"
-                             "HINT: please use any of the following "
-                             "tablespaces: %s"
-                             % (item, ', '.join(valid_tablespaces)))
+            output.error("Invalid tablespace name '%s'\n"
+                         "HINT: Please use any of the following "
+                         "tablespaces: %s",
+                         item, ', '.join(valid_tablespaces))
+            output.close_and_exit()
 
     # explicitly disallow the rsync remote syntax (common mistake)
     if ':' in args.destination_directory:
-        raise SystemExit(
-            "ERROR: the destination directory parameter cannot contain the ':' character\n"
-            "HINT: if you want to do a remote recovery you have to use the --remote-ssh-command option")
+        output.error(
+            "The destination directory parameter "
+            "cannot contain the ':' character\n"
+            "HINT: If you want to do a remote recovery you have to use "
+            "the --remote-ssh-command option")
+        output.close_and_exit()
     if args.retry_sleep is not None:
         server.config.basebackup_retry_sleep = args.retry_sleep
     if args.retry_times is not None:
         server.config.basebackup_retry_times = args.retry_times
-    for line in server.recover(backup,
-                               args.destination_directory,
-                               tablespaces=tablespaces,
-                               target_tli=args.target_tli,
-                               target_time=args.target_time,
-                               target_xid=args.target_xid,
-                               target_name=args.target_name,
-                               exclusive=args.exclusive,
-                               remote_command=args.remote_ssh_command):
+    server.recover(backup,
+                   args.destination_directory,
+                   tablespaces=tablespaces,
+                   target_tli=args.target_tli,
+                   target_time=args.target_time,
+                   target_xid=args.target_xid,
+                   target_name=args.target_name,
+                   exclusive=args.exclusive,
+                   remote_command=args.remote_ssh_command)
 
-        yield line
+    output.close_and_exit()
 
 
 @named('show-server')
@@ -420,16 +426,17 @@ def list_files(args):
     """
     server = get_server(args)
     if server is None:
-        yield "Unknown server '%s'" % (args.server_name)
-        return
+        output.error("Unknown server '%s'", args.server_name)
+        output.close_and_exit()
     # Retrieves the backup
     backup = parse_backup_id(server, args)
     if backup is None:
-        yield "Unknown backup '%s' for server '%s'" % (
-            args.backup_id, args.server_name)
-        return
+        output.error("Unknown backup '%s' for server '%s'", args.backup_id,
+                     args.server_name)
+        output.close_and_exit()
     for line in backup.get_list_of_files(args.target):
-        yield line
+        output.info(line, log=False)
+    output.close_and_exit()
 
 
 @arg('server_name',
@@ -444,34 +451,16 @@ def delete(args):
     """
     server = get_server(args)
     if server is None:
-        yield "Unknown server '%s'" % (args.server_name)
-        return
+        output.error("Unknown server '%s'", args.server_name)
+        output.close_and_exit()
     # Retrieves the backup
     backup = parse_backup_id(server, args)
     if backup is None:
-        yield "Unknown backup '%s' for server '%s'" % (
-            args.backup_id, args.server_name)
-        return
-    for line in server.delete_backup(backup):
-        yield line
-
-
-class stream_wrapper(object):
-    """ This class represents a wrapper for a stream
-    """
-
-    def __init__(self, stream):
-        self.stream = stream
-
-    def set_stream(self, stream):
-        ''' Set the stream as stream argument '''
-        self.stream = stream
-
-    def __getattr__(self, attr):
-        return getattr(self.stream, attr)
-
-
-_output_stream = stream_wrapper(sys.stdout)
+        output.error("Unknown backup '%s' for server '%s'", args.backup_id,
+                     args.server_name)
+        output.close_and_exit()
+    server.delete_backup(backup)
+    output.close_and_exit()
 
 
 def global_config(args):
@@ -519,10 +508,6 @@ def global_config(args):
 
     _logger.debug('Initialized Barman version %s (config: %s)',
                   barman.__version__, config.config_file)
-    if hasattr(args, 'quiet') and args.quiet:
-        _logger.debug("Replacing output stream")
-        global _output_stream
-        _output_stream.set_stream(open(os.devnull, 'w'))
 
 
 def get_server(args):
@@ -614,7 +599,7 @@ def main():
     )
     # noinspection PyBroadException
     try:
-        p.dispatch(pre_call=global_config, output_file=_output_stream)
+        p.dispatch(pre_call=global_config)
     except KeyboardInterrupt:
         msg = "Process interrupted by user (KeyboardInterrupt)"
         output.exception(msg)
@@ -631,6 +616,7 @@ if __name__ == '__main__':
     # This code requires the mock module and allow us to test
     # bash completion inside the IDE debugger
     try:
+        # noinspection PyUnresolvedReferences
         import mock
         sys.stdout = mock.Mock(wraps=sys.stdout)
         sys.stdout.isatty.return_value = True
