@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2014 2ndQuadrant Italia (Devise.IT S.r.L.)
+# Copyright (C) 2013-2015 2ndQuadrant Italia (Devise.IT S.r.L.)
 #
 # This file is part of Barman.
 #
@@ -125,6 +125,21 @@ def _dispatch(obj, prefix, name, *args, **kwargs):
         raise ValueError("The object %r does not have the %r method" % (
             obj, method_name))
 
+def is_quiet():
+    """
+    Calls the "is_quiet" method, accessing the protected parameter _quiet
+    of the instanced OutputWriter
+    :return bool: the _quiet parameter value
+    """
+    return _writer.is_quiet()
+
+def is_debug():
+    """
+    Calls the "is_debug" method, accessing the protected parameter _debug
+    of the instanced OutputWriter
+    :return bool: the _debug parameter value
+    """
+    return _writer.is_debug()
 
 def debug(message, *args, **kwargs):
     """
@@ -311,6 +326,22 @@ class ConsoleOutputWriter(object):
         """
         print >> sys.stderr, _format_message(message, args)
 
+    def is_quiet(self):
+        """
+        Access the quiet property of the OutputWriter instance
+
+        :return bool: if the writer is quiet or not
+        """
+        return self._quiet
+
+    def is_debug(self):
+        """
+        Access the debug property of the OutputWriter instance
+
+        :return bool: if the writer is in debug mode or not
+        """
+        return self._debug
+
     def debug(self, message, *args):
         """
         Emit debug.
@@ -475,9 +506,17 @@ class ConsoleOutputWriter(object):
                               item.name, item.location, item.oid)
             self.info("")
             self.info("  Base backup information:")
-            self.info("    Disk usage           : %s",
+            self.info("    Disk usage           : %s (%s with WALs)",
+                      pretty_size(data['size']),
                       pretty_size(data['size'] + data[
                           'wal_size']))
+            if data['deduplicated_size'] is not None and data['size'] > 0:
+                deduplication_ratio = 1 - (float(data['deduplicated_size']) /
+                                           data['size'])
+                self.info("    Incremental size     : %s (-%s)",
+                          pretty_size(data['deduplicated_size']),
+                          '{percent:.2%}'.format(percent=deduplication_ratio)
+                          )
             self.info("    Timeline             : %s", data['timeline'])
             self.info("    Begin WAL            : %s",
                       data['begin_wal'])
