@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2014 2ndQuadrant Italia (Devise.IT S.r.L.)
+# Copyright (C) 2011-2015 2ndQuadrant Italia (Devise.IT S.r.L.)
 #
 # This file is part of Barman.
 #
@@ -26,9 +26,7 @@ import logging
 from contextlib import contextmanager
 
 import psycopg2
-import sys
-import shutil
-from datetime import timedelta
+from psycopg2.extras import RealDictCursor
 
 from barman import output
 from barman.config import BackupOptions
@@ -46,6 +44,24 @@ import xlog
 _logger = logging.getLogger(__name__)
 
 
+class SshCommandException(Exception):
+    """
+    Error parsing ssh_command parameter
+    """
+
+
+class ConninfoException(Exception):
+    """
+    Error parsing conninfo parameter
+    """
+
+
+class PostgresConnectionError(Exception):
+    """
+    Error connecting to PostgreSQL server.
+    """
+
+
 class Server(object):
     """
     This class represents a server to backup
@@ -53,17 +69,27 @@ class Server(object):
     XLOG_DB = "xlog.db"
 
     def __init__(self, config):
-        ''' The Constructor.
+        """ The Constructor.
 
         :param config: the server configuration
-        '''
+        """
         self.config = config
         self.conn = None
         self.server_txt_version = None
         self.server_version = None
-        self.ssh_options = config.ssh_command.split()
+        try:
+            self.ssh_options = config.ssh_command.split()
+        except AttributeError:
+            raise SshCommandException(
+                'Missing or invalid ssh_command in barman configuration '
+                'for server %s' % config.name)
         self.ssh_command = self.ssh_options.pop(0)
-        self.ssh_options.extend("-o BatchMode=yes -o StrictHostKeyChecking=no".split())
+        self.ssh_options.extend("-o BatchMode=yes "
+                                "-o StrictHostKeyChecking=no".split())
+        if self.config.conninfo is None:
+            raise ConninfoException(
+                'Missing conninfo parameter in barman configuration '
+                'for server %s' % config.name)
         self.backup_manager = BackupManager(self)
         self.configuration_files = None
         self.enforce_retention_policies = False
@@ -108,7 +134,6 @@ class Server(object):
         # Initialise retention policies
         self._init_retention_policies()
 
-
     def _init_retention_policies(self):
 
         # Set retention policy mode
@@ -193,8 +218,11 @@ class Server(object):
         """
         Checks PostgreSQL connection
         """
-        remote_status = self.get_remote_status()
-        if remote_status['server_txt_version']:
+        try:
+            remote_status = self.get_remote_status()
+        except PostgresConnectionError as e:
+            remote_status = None
+        if remote_status is not None and remote_status['server_txt_version']:
             output.result('check', self.config.name, 'PostgreSQL', True)
         else:
             output.result('check', self.config.name, 'PostgreSQL', False)
@@ -215,9 +243,17 @@ class Server(object):
         else:
             output.result('check', self.config.name, 'archive_mode', False,
                           "please set it to 'on'")
-        if remote_status['archive_command'] and\
+        if remote_status['archive_command'] and \
                 remote_status['archive_command'] != '(disabled)':
             output.result('check', self.config.name, 'archive_command', True)
+            # Report if the archiving process works without issues.
+            # Skip if the archive_command check fails
+            # It can be None if PostgreSQL is older than 9.4
+            if remote_status.get('is_archiving') is not None:
+                output.result('check',
+                              self.config.name,
+                              'continuous archiving',
+                              remote_status['is_archiving'])
         else:
             output.result('check', self.config.name, 'archive_command', False,
                           'please set it accordingly to documentation')
@@ -262,7 +298,7 @@ class Server(object):
         """
         # first check: check backup maximum age
         if self.config.last_backup_maximum_age is not None:
-            # get maximum age informations
+            # get maximum age information
             backup_age = self.backup_manager.validate_last_backup_maximum_age(
                 self.config.last_backup_maximum_age)
 
@@ -312,17 +348,36 @@ class Server(object):
                       "PostgreSQL 'archive_command' setting",
                       remote_status['archive_command']
                       or "FAILED (please set it accordingly to documentation)")
+        last_wal = str(remote_status['last_archived_wal'])
+        # If PostgreSQL is >= 9.4 we have the last_archived_time
+        if remote_status.get('last_archived_time'):
+                last_wal += ", at %s" % (
+                    remote_status['last_archived_time'].ctime())
         output.result('status', self.config.name,
-                      "last_shipped_wal",
-                      "Archive status",
-                      "last shipped WAL segment %s" %
-                      remote_status['last_shipped_wal']
-                      or "No WAL segment shipped yet")
+                      "last_archived_wal",
+                      "Last archived WAL",
+                      last_wal or "No WAL segment shipped yet")
         if remote_status['current_xlog']:
             output.result('status', self.config.name,
                           "current_xlog",
                           "Current WAL segment",
                           remote_status['current_xlog'])
+        # Set output for WAL archive failures (PostgreSQL >= 9.4)
+        if remote_status.get('failed_count') is not None:
+            remote_fail = str(remote_status['failed_count'])
+            if int(remote_status['failed_count']) > 0:
+                remote_fail += " (%s at %s)" % (
+                    remote_status['last_failed_wal'],
+                    remote_status['last_failed_time'].ctime())
+            output.result('status', self.config.name, 'failed_count',
+                          'Failures of WAL archiver', remote_fail)
+        # Add hourly archive rate if available (PostgreSQL >= 9.4) and > 0
+        if remote_status.get('current_archived_wals_per_second'):
+            output.result(
+                'status', self.config.name,
+                'server_archived_wals_per_hour',
+                'Server WAL archiving rate', '%0.2f/hour' % (
+                    3600 * remote_status['current_archived_wals_per_second']))
 
     def status_retention_policies(self):
         """
@@ -373,10 +428,46 @@ class Server(object):
                     return True
                 else:
                     return False
-        except psycopg2.Error, e:
+        except (PostgresConnectionError, psycopg2.Error) as e:
             _logger.debug("Error retrieving pgespresso information: %s", e)
             return False
 
+    def get_pg_stat_archiver(self):
+        """
+        This method gathers statistics from pg_stat_archiver
+        (postgres 9.4+ or greater required)
+
+        :return dict|None: a dictionary containing Postgres statistics from
+            pg_stat_archiver or None
+        """
+        try:
+            with self.pg_connect() as conn:
+                # pg_stat_archiver is only available from Postgres 9.4+
+                if self.server_version < 90400:
+                    return None
+                cur = conn.cursor(cursor_factory=RealDictCursor)
+                # Select from pg_stat_archiver statistics view,
+                # retrieving statistics about WAL archiver process activity,
+                # also evaluating if the server is archiving without issues
+                # and the archived WALs per second rate
+                cur.execute(
+                    "SELECT *, current_setting('archive_mode')::BOOLEAN "
+                    "AND (last_failed_wal IS NULL "
+                    "OR last_failed_wal <= last_archived_wal) "
+                    "AS is_archiving, "
+                    "CAST (archived_count AS NUMERIC) "
+                    "/ EXTRACT (EPOCH FROM age(now(), stats_reset)) "
+                    "AS current_archived_wals_per_second "
+                    "FROM pg_stat_archiver")
+                q_result = cur.fetchone()
+                if q_result:
+                    return q_result
+                else:
+                    return None
+        except (PostgresConnectionError, psycopg2.Error) as e:
+            _logger.debug("Error retrieving pg_stat_archive data: %s", e)
+            return None
+
     def pg_is_in_recovery(self):
         """
         Returns true if PostgreSQL server is in recovery mode
@@ -393,7 +484,7 @@ class Server(object):
                     return True
                 else:
                     return False
-        except psycopg2.Error, e:
+        except (PostgresConnectionError, psycopg2.Error) as e:
             _logger.debug("Error calling pg_is_in_recovery() function: %s", e)
             return None
 
@@ -424,7 +515,6 @@ class Server(object):
                     _logger.debug(
                         "Error retrieving PostgreSQL version: %s", e)
 
-
                 result['pgespresso_installed'] = self.pg_espresso_installed()
 
                 try:
@@ -438,39 +528,47 @@ class Server(object):
                     _logger.debug("Error retrieving current xlog: %s", e)
 
                 result.update(self.get_pg_configuration_files())
-        except psycopg2.Error, e:
-            _logger.warn("Error retrieving PostgreSQL status: %s", e)
 
-        # TODO: replace with RemoteUnixCommand
-        cmd = Command(self.ssh_command, self.ssh_options)
-        result['last_shipped_wal'] = None
-        if result['data_directory'] and result['archive_command']:
+                # Add pg_stat_archiver statistics if the view is supported
+                pg_stat_archiver = self.get_pg_stat_archiver()
+                if pg_stat_archiver is not None:
+                    result.update(pg_stat_archiver)
+
+        except (PostgresConnectionError, psycopg2.Error) as e:
+            _logger.warn("Error retrieving PostgreSQL status: %s", e)
+        # If last_archived_wal is not available from pg_stat_archiver
+        # retrieve it listing the $PGDATa/pg_xlog/archive_status directory
+        if result.get('last_archived_wal') is None \
+                and result['data_directory'] \
+                and result['archive_command']:
+            # TODO: replace with RemoteUnixCommand
+            cmd = Command(self.ssh_command, self.ssh_options)
             archive_dir = os.path.join(result['data_directory'],
                                        'pg_xlog', 'archive_status')
             out = str(cmd.getoutput('ls', '-tr', archive_dir)[0])
             for line in out.splitlines():
                 if line.endswith('.done'):
                     name = line[:-5]
-                    if xlog.is_wal_file(name):
-                        result['last_shipped_wal'] = line[:-5]
+                    if xlog.is_any_xlog_file(name):
+                        result['last_archived_wal'] = line[:-5]
         return result
 
     def show(self):
         """
         Shows the server configuration
         """
-        #Populate result map with all the required keys
+        # Populate result map with all the required keys
         result = dict([
             (key, getattr(self.config, key))
             for key in self.config.KEYS
         ])
         remote_status = self.get_remote_status()
         result.update(remote_status)
-        # backup maximum age section
+        # Backup maximum age section
         if self.config.last_backup_maximum_age is not None:
             age = self.backup_manager.validate_last_backup_maximum_age(
                 self.config.last_backup_maximum_age)
-            # if latest backup is between the limits of the
+            # If latest backup is between the limits of the
             # last_backup_maximum_age configuration, display how old is
             # the latest backup.
             if age[0]:
@@ -479,7 +577,7 @@ class Server(object):
                         self.config.last_backup_maximum_age),
                      age[1])
             else:
-                # if latest backup is outside the limits of the
+                # If latest backup is outside the limits of the
                 # last_backup_maximum_age configuration (or the configuration
                 # value is none), warn the user.
                 msg = "%s (WARNING! latest backup is %s old)" % \
@@ -497,15 +595,21 @@ class Server(object):
         """
         A generic function to connect to Postgres using Psycopg2
         """
-        myconn = self.conn == None
+        myconn = self.conn is None
         if myconn:
-            self.conn = psycopg2.connect(self.config.conninfo)
-            self.server_version = self.conn.server_version
-            if (self.server_version >= 90000
-                    and 'application_name=' not in self.config.conninfo):
-                cur = self.conn.cursor()
-                cur.execute('SET application_name TO barman')
-                cur.close()
+            try:
+                self.conn = psycopg2.connect(self.config.conninfo)
+                self.server_version = self.conn.server_version
+                if (self.server_version >= 90000
+                        and 'application_name=' not in self.config.conninfo):
+                    cur = self.conn.cursor()
+                    cur.execute('SET application_name TO barman')
+                    cur.close()
+            # If psycopg2 fails to connect to the host,
+            # raise the appropriate exception
+            except psycopg2.DatabaseError as e:
+                raise PostgresConnectionError("Cannot connect to postgres: %s"
+                                              % e)
         try:
             yield self.conn
         finally:
@@ -519,22 +623,24 @@ class Server(object):
 
         :param name: a parameter name
         """
-        with self.pg_connect() as conn:
-            try:
+
+        try:
+            with self.pg_connect() as conn:
                 cur = conn.cursor()
                 cur.execute('SHOW "%s"' % name.replace('"', '""'))
                 return cur.fetchone()[0]
-            except psycopg2.Error, e:
-                _logger.debug("Error retrieving PostgreSQL setting '%s': %s",
-                              name.replace('"', '""'), e)
-                return None
+        except (PostgresConnectionError, psycopg2.Error) as e:
+            _logger.debug("Error retrieving PostgreSQL setting '%s': %s",
+                          name.replace('"', '""'), e)
+            return None
 
     def get_pg_tablespaces(self):
         """
         Returns a list of tablespaces or None if not present
         """
-        with self.pg_connect() as conn:
-            try:
+
+        try:
+            with self.pg_connect() as conn:
                 cur = conn.cursor()
                 if self.server_version >= 90200:
                     cur.execute(
@@ -548,9 +654,9 @@ class Server(object):
                         "FROM pg_tablespace WHERE spclocation != ''")
                 # Generate a list of tablespace objects
                 return [Tablespace._make(item) for item in cur.fetchall()]
-            except psycopg2.Error, e:
-                _logger.debug("Error retrieving PostgreSQL tablespaces: %s", e)
-                return None
+        except (PostgresConnectionError, psycopg2.Error) as e:
+            _logger.debug("Error retrieving PostgreSQL tablespaces: %s", e)
+            return None
 
     def get_pg_configuration_files(self):
         """
@@ -558,8 +664,8 @@ class Server(object):
         """
         if self.configuration_files:
             return self.configuration_files
-        with self.pg_connect() as conn:
-            try:
+        try:
+            with self.pg_connect() as conn:
                 cur = conn.cursor()
                 cur.execute("SELECT name, setting FROM pg_settings "
                             "WHERE name IN ("
@@ -568,10 +674,10 @@ class Server(object):
                 for cname, cpath in cur.fetchall():
                     self.configuration_files[cname] = cpath
                 return self.configuration_files
-            except psycopg2.Error, e:
-                _logger.debug("Error retrieving PostgreSQL configuration files "
-                              "location: %s", e)
-                return {}
+        except (PostgresConnectionError, psycopg2.Error) as e:
+            _logger.debug("Error retrieving PostgreSQL configuration files "
+                          "location: %s", e)
+            return {}
 
     def pg_start_backup(self, backup_label):
         """
@@ -696,8 +802,6 @@ class Server(object):
         """
         stop backup wrapper
 
-        :param backup_label: label for the backup
-        :param immediate_checkpoint: Boolean for immediate checkpoint execution
         :param backup_info: backup_info object
         :return:
         """
@@ -741,8 +845,6 @@ class Server(object):
     def backup(self):
         """
         Performs a backup for the server
-
-        :param immediate_checkpoint: Boolean for immediate checkpoint execution
         """
         try:
             # check required backup directories exist
@@ -759,6 +861,8 @@ class Server(object):
             # lock acquisition and backup execution
             with LockFile(filename, raise_if_fail=True):
                 self.backup_manager.backup()
+            # Archive incoming WALs and update WAL catalogue through cron
+            self.cron(verbose=False)
 
         except LockFileBusy:
             output.error("Another backup process is running")
@@ -858,12 +962,17 @@ class Server(object):
         with self.xlogdb() as fxlogdb:
             for line in fxlogdb:
                 wal_info = WalFileInfo.from_xlogdb_line(self, line)
+                # Handle .history files: add all of them to the output,
+                # regardless of their age
+                if xlog.is_history_file(wal_info.name):
+                    yield wal_info
+                    continue
                 if wal_info.name < begin:
                     continue
                 tli, _, _ = xlog.decode_segment_name(wal_info.name)
                 if tli > target_tli:
                     continue
-                yield wal_info.name
+                yield wal_info
                 if wal_info.name > end:
                     end = wal_info.name
                     if target_time and target_time < wal_info.time:
@@ -872,15 +981,17 @@ class Server(object):
             for line in fxlogdb:
                 wal_info = WalFileInfo.from_xlogdb_line(self, line)
                 if xlog.is_history_file(wal_info.name):
-                    yield wal_info.name
+                    yield wal_info
 
     # TODO: merge with the previous
-    def get_wal_until_next_backup(self, backup):
+    def get_wal_until_next_backup(self, backup, include_history=False):
         """
         Get the xlog files between backup and the next
 
         :param BackupInfo backup: a backup object, the starting point
             to retrieve WALs
+        :param bool include_history: option for the inclusion of
+            include_history files into the output
         """
         begin = backup.begin_wal
         next_end = None
@@ -891,6 +1002,13 @@ class Server(object):
         with self.xlogdb() as fxlogdb:
             for line in fxlogdb:
                 wal_info = WalFileInfo.from_xlogdb_line(self, line)
+                # Handle .history files: add all of them to the output,
+                # regardless of their age, if requested (the 'include_history'
+                # parameter is True)
+                if xlog.is_history_file(wal_info.name):
+                    if include_history:
+                        yield wal_info
+                    continue
                 if wal_info.name < begin:
                     continue
                 tli, _, _ = xlog.decode_segment_name(wal_info.name)
@@ -951,8 +1069,11 @@ class Server(object):
                 wal_info['wal_last'] = item.name
                 wal_info['wal_last_timestamp'] = item.time
 
-        # Estimate WAL ratio
-        if wal_info['wal_last_timestamp']:
+        # Calculate statistics only for complete backups
+        # If the cron is not running for any reason, the required
+        # WAL files could be missing
+        if wal_info['wal_first'] and wal_info['wal_last']:
+            # Estimate WAL ratio
             # Calculate the difference between the timestamps of
             # the first WAL (begin of backup) and the last WAL
             # associated to the current backup
@@ -962,25 +1083,25 @@ class Server(object):
                 wal_info['wals_per_second'] = (float(wal_info['wal_num']) /
                                                wal_info['wal_total_seconds'])
 
-        # evaluation of compression ratio for basebackup WAL files
-        wal_info['wal_theoretical_size'] = \
-            wal_info['wal_num'] * float(xlog.XLOG_SEG_SIZE)
-        try:
-            wal_info['wal_compression_ratio'] = 1 - (
-                wal_info['wal_size'] /
-                wal_info['wal_theoretical_size'])
-        except ZeroDivisionError:
-            wal_info['wal_compression_ratio'] = 0.0
+            # evaluation of compression ratio for basebackup WAL files
+            wal_info['wal_theoretical_size'] = \
+                wal_info['wal_num'] * float(xlog.XLOG_SEG_SIZE)
+            try:
+                wal_info['wal_compression_ratio'] = 1 - (
+                    wal_info['wal_size'] /
+                    wal_info['wal_theoretical_size'])
+            except ZeroDivisionError:
+                wal_info['wal_compression_ratio'] = 0.0
 
-        # evaluation of compression ratio of WAL files
-        wal_info['wal_until_next_theoretical_size'] = \
-            wal_info['wal_until_next_num'] * float(xlog.XLOG_SEG_SIZE)
-        try:
-            wal_info['wal_until_next_compression_ratio'] = 1 - (
-                wal_info['wal_until_next_size'] /
-                wal_info['wal_until_next_theoretical_size'])
-        except ZeroDivisionError:
-            wal_info['wal_until_next_compression_ratio'] = 0.0
+            # evaluation of compression ratio of WAL files
+            wal_info['wal_until_next_theoretical_size'] = \
+                wal_info['wal_until_next_num'] * float(xlog.XLOG_SEG_SIZE)
+            try:
+                wal_info['wal_until_next_compression_ratio'] = 1 - (
+                    wal_info['wal_until_next_size'] /
+                    wal_info['wal_until_next_theoretical_size'])
+            except ZeroDivisionError:
+                wal_info['wal_until_next_compression_ratio'] = 0.0
 
         return wal_info
 
@@ -991,12 +1112,16 @@ class Server(object):
     def cron(self, verbose=True):
         """
         Maintenance operations
+
+        :param bool verbose: report even if no actions
         """
         filename = os.path.join(self.config.barman_home,
                                 '.%s-cron.lock' % self.config.name)
         try:
             with LockFile(filename, raise_if_fail=True, wait=True):
-                return self.backup_manager.cron(verbose=verbose)
+                self.backup_manager.cron(verbose=verbose)
+                # Retention policy management
+                self.backup_manager.cron_retention_policy()
         except LockFilePermissionDenied, e:
             output.error("Permission denied, unable to access '%s'" % e)
         except (OSError, IOError), e:
@@ -1023,7 +1148,7 @@ class Server(object):
         xlogdb = os.path.join(self.config.wals_directory, self.XLOG_DB)
 
         xlogdb_lock = xlogdb + ".lock"
-        with LockFile(xlogdb_lock, wait=True):
+        with LockFile(xlogdb_lock, raise_if_fail=True, wait=True):
             # If the file doesn't exist and it is required to read it,
             # we open it in a+ mode, to be sure it will be created
             if not os.path.exists(xlogdb) and mode.startswith('r'):
